/*************************************************************************
 *
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
 *
 * Copyright 2008 by Sun Microsystems, Inc.
 *
 * OpenOffice.org - a multi-platform office productivity suite
 *
 * $RCSfile: JmolFrame.java,v $
 * $Revision: 1.1 $
 *
 * This file is part of OpenOffice.org.
 *
 * OpenOffice.org is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License version 3
 * only, as published by the Free Software Foundation.
 *
 * OpenOffice.org is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License version 3 for more details
 * (a copy is included in the LICENSE file that accompanied this code).
 *
 * You should have received a copy of the GNU Lesser General Public License
 * version 3 along with OpenOffice.org.  If not, see
 * <http://www.openoffice.org/license.html>
 * for a copy of the LGPLv3 License.
 *
 ************************************************************************/

package org.openoffice.embed.jmol;

import java.awt.*;
import java.awt.event.*;
import java.awt.image.*;
import javax.swing.JFrame;
import java.io.*;
import javax.imageio.ImageIO;
import javax.swing.JFileChooser;
import javax.swing.JMenuItem;
import javax.swing.JPanel;
import javax.swing.JPopupMenu;
import org.jmol.adapter.smarter.SmarterJmolAdapter;
import org.jmol.api.JmolAdapter;
import org.jmol.viewer.Viewer;

public class JmolFrame extends JFrame {

    JmolEmbeddedObject m_aEmbObj;
    JmolPanel m_aJmolPanel;
    String m_aJmolStrModel;
    private java.util.Properties m_aProperties;
    private Dimension m_aSize;
    private Point m_aPosition;
    BufferedImage m_aBufImage;
    public static byte[] m_aDefaultImage = null;

    public JmolFrame(
                        String sName,
                        JmolEmbeddedObject aEmbObj,
                        /*int nWidth, int nHeight,*/
                        java.util.Properties aProperties) {
        super(sName);
        if (aProperties == null) {
            try {
                java.io.InputStream inStream = JmolFrame.class.getResourceAsStream(
                        JmolConstants.JMOL_PROPERTIES_FILE);
                aProperties = new java.util.Properties();
                aProperties.load(inStream);
            } catch (Exception e) {
                e.printStackTrace();
            }
        }
        m_aProperties = aProperties;
        m_aSize = getSizeFromProperties();
        m_aEmbObj = aEmbObj;

        Container contentPane = getContentPane();
        m_aJmolPanel = new JmolPanel();
        contentPane.add(m_aJmolPanel);
        createPopupMenu(m_aJmolPanel);

        pack();

        setSize(m_aSize);
        setVisible(false);

        addWindowListener(new JmolApplicationCloser());

        if (m_aDefaultImage == null) {
            byte[] image = null;
            try {
                InputStream in = JmolFrame.class.getResourceAsStream(
                        JmolConstants.JMOL_DEFAULT_IMAGE_FILE);
                int len = in.available();
                image = new byte[len];
                in.read(image);
                in.close();
            } catch (Exception e) {
                e.printStackTrace();
            }
            m_aDefaultImage = image;
        }

        updateJmolState();
        setLocation(
                Integer.valueOf(m_aProperties.getProperty(JmolConstants.JMOL_WINDOW_X_PROPERTY)),
                Integer.valueOf(m_aProperties.getProperty(JmolConstants.JMOL_WINDOW_Y_PROPERTY))
        );
    }

    public void updateJmolState() {
        m_aJmolStrModel = m_aProperties.getProperty(JmolConstants.JMOL_MODEL_PROPERTY);
        setModelInline(m_aJmolStrModel);
        sendScriptCommand(JmolConstants.JMOL_WINDOW_STATE_PROPERTY);
        sendScriptCommand(JmolConstants.JMOL_VARIABLE_STATE_PROPERTY);
        sendScriptCommand(JmolConstants.JMOL_PERSPECTIVE_STATE_PROPERTY);
        sendScriptCommand(JmolConstants.JMOL_SELECTION_STATE_PROPERTY);
        sendScriptCommand(JmolConstants.JMOL_MODEL_STATE_PROPERTY);
    }

    public void updateVisibleState(){

    }

    public String getPDBString() {
        return m_aJmolStrModel != null &&  !m_aJmolStrModel.isEmpty()
                ? m_aJmolStrModel : getDefaultPDB();
    }

    public void setPDBString(String aText) {
        m_aProperties.setProperty(JmolConstants.JMOL_MODEL_PROPERTY, aText);
        m_aJmolStrModel = aText;
    }

    //*************************************************************************

    public Dimension getSizeFromProperties(){
        return new Dimension(
                Integer.valueOf(m_aProperties.getProperty(JmolConstants.JMOL_WINDOW_WIDTH_PROPERTY)),
                Integer.valueOf(m_aProperties.getProperty(JmolConstants.JMOL_WINDOW_HEIGHT_PROPERTY)));
    }

    public Dimension getAppSize() {
        return !isShowing() ? getSizeFromProperties() : getSize();
    }

    public void setAppSize(Dimension aSize) {
        Dimension aOwnSize = getSize();
        Dimension aAppSize = m_aJmolPanel.getSize();
        Dimension aToSet =
                new Dimension(
                (int) (aSize.getWidth() + aOwnSize.getWidth() - aAppSize.getWidth()),
                (int) (aSize.getHeight() + aOwnSize.getHeight() - aAppSize.getHeight()));

        setSize(aToSet);
        validate();

    // pack();
    }

    //*************************************************************************

    public byte[] getReplacementImage() {
        Dimension aDim = m_aJmolPanel.getSize();
        BufferedImage aBufImage = null;

        if (m_aBufImage != null) {
            aBufImage = m_aBufImage;
        } else {
            try {
                int nWidth = (int) aDim.getWidth();
                int nHeight = (int) aDim.getHeight();
                aBufImage = new BufferedImage(nWidth, nHeight, BufferedImage.TYPE_INT_RGB);
                Graphics2D aGr = aBufImage.createGraphics();
                aGr.setBackground(Color.WHITE);
                aGr.clearRect(0, 0, nWidth, nHeight);
                aGr.dispose();
            } catch (java.lang.Exception e) {
            }
        }

        if (aBufImage != null) {
            try {
                File aTmpFile = File.createTempFile("temp", ".png");
                ImageIO.write(aBufImage, "png", aTmpFile);

                int nLen = (int) aTmpFile.length();
                byte[] aResult = new byte[nLen];
                FileInputStream aTmpStream = new FileInputStream(aTmpFile);
                aTmpStream.read(aResult);
                aTmpStream.close();
                aTmpFile.delete();

                return aResult;
            } catch (java.lang.Exception e) {
            }
        }

        return new byte[0];
    }

    protected java.util.Properties getProperties(){
        return m_aProperties;
    }

    private void sendScriptCommand(String sPropName) {
        String str = m_aProperties.getProperty(sPropName);
        if (str != null && !str.isEmpty()) {
            try {
                m_aJmolPanel.getViewer().evalString(str);
            } catch (Exception e) {
                e.printStackTrace();
            }
        }
    }

    private void setModelInline(String sModel) {
        if (sModel != null) {
            m_aJmolPanel.getViewer().openStringInline(sModel);
        }
    }

    private void storeStateInfo() {
        m_aProperties.setProperty( JmolConstants.JMOL_WINDOW_WIDTH_PROPERTY,
                String.valueOf(m_aJmolPanel.getWidth()));
        m_aProperties.setProperty( JmolConstants.JMOL_WINDOW_HEIGHT_PROPERTY,
                String.valueOf(m_aJmolPanel.getHeight()));
        m_aProperties.setProperty( JmolConstants.JMOL_WINDOW_X_PROPERTY,
                String.valueOf(m_aJmolPanel.getX()));
        m_aProperties.setProperty( JmolConstants.JMOL_WINDOW_Y_PROPERTY,
                String.valueOf(m_aJmolPanel.getY()));
        m_aProperties.setProperty( JmolConstants.JMOL_MODEL_PROPERTY,
                m_aJmolStrModel);
        m_aProperties.setProperty( JmolConstants.JMOL_WINDOW_STATE_PROPERTY,
                m_aJmolPanel.getViewer().getStateInfo(JmolConstants.JMOL_WINDOW_STATE));
        m_aProperties.setProperty( JmolConstants.JMOL_VARIABLE_STATE_PROPERTY,
                m_aJmolPanel.getViewer().getStateInfo(JmolConstants.JMOL_VARIABLE_STATE));
        m_aProperties.setProperty( JmolConstants.JMOL_MODEL_STATE_PROPERTY,
                m_aJmolPanel.getViewer().getStateInfo(JmolConstants.JMOL_MODEL_STATE));
        m_aProperties.setProperty( JmolConstants.JMOL_PERSPECTIVE_STATE_PROPERTY,
                m_aJmolPanel.getViewer().getStateInfo(JmolConstants.JMOL_PERSPECTIVE_STATE));
        m_aProperties.setProperty( JmolConstants.JMOL_SELECTION_STATE_PROPERTY,
                m_aJmolPanel.getViewer().getStateInfo(JmolConstants.JMOL_SELECTION_STATE));
    }

    //*************************************************************************

    class JmolApplicationCloser extends WindowAdapter {

        @Override
        public void windowOpened(WindowEvent ev) {
            try {
                setSize(m_aSize);// really needed?
                updateJmolState();// really needed?
            } catch (Exception e) {
                e.printStackTrace();
            }
        }

        @Override
        public void windowClosing(WindowEvent e) {
            m_aBufImage = new BufferedImage(
                    m_aJmolPanel.getWidth(),
                    m_aJmolPanel.getHeight(),
                    BufferedImage.TYPE_INT_RGB);
            Graphics2D aGr = m_aBufImage.createGraphics();
            m_aJmolPanel.paintAll(aGr);
            aGr.dispose();

            m_aSize = getSize(m_aSize);
            m_aPosition = new Point(getX(),getY());
            // this must be done before setting it not visible
            // so we can store the real size and position
            storeStateInfo();
            setVisible(false);

            m_aEmbObj.CloseFrameRequest();
        }
    }
    class JmolPanel extends JPanel {

        Viewer viewer;
        JmolAdapter adapter;

        JmolPanel() {
            adapter = new SmarterJmolAdapter();
            viewer = (Viewer) Viewer.allocateViewer(this, adapter);
        }

        public Viewer getViewer() {
            return viewer;
        }
        final Dimension currentSize = new Dimension();
        final Rectangle rectClip = new Rectangle();

        @Override
        public void paint(Graphics g) {
            getSize(currentSize);
            g.getClipBounds(rectClip);
            viewer.renderScreenImage(g, currentSize, rectClip);
        }
    }

    //*************************************************************************

    class PopupListener extends MouseAdapter {
        JPopupMenu popup;

        PopupListener(JPopupMenu popupMenu) {
            popup = popupMenu;
        }

        @Override
        public void mousePressed(MouseEvent e) {
            maybeShowPopup(e);
        }

        @Override
        public void mouseReleased(MouseEvent e) {
            maybeShowPopup(e);
        }

        private void maybeShowPopup(MouseEvent e) {
            if (e.isPopupTrigger()) {
                popup.show(e.getComponent(),
                           e.getX(), e.getY());
            }
        }
    }


    private void createPopupMenu(JmolPanel panel) {

        ActionLsnr actlistener = new ActionLsnr();
        JMenuItem menuItem;

        JPopupMenu popup = new JPopupMenu();
        menuItem = new JMenuItem("Open PDB file");
        menuItem.addActionListener(actlistener);
        popup.add(menuItem);

        MouseListener popupListener = new PopupListener(popup);
        panel.addMouseListener(popupListener);
    }

    class ActionLsnr implements ActionListener {

        public void actionPerformed(ActionEvent e) {

            final JFileChooser fileDlg = new JFileChooser();

            fileDlg.setMultiSelectionEnabled(false);
            fileDlg.setFileFilter(new javax.swing.filechooser.FileFilter() {
                @Override
                public boolean accept(File f) {
                    boolean bRet = false;
                    if (f.isDirectory())
                        return true;
                    else {
                        String ext = null;
                        String s = f.getName();
                        int i = s.lastIndexOf('.');
                        if (i > 0 && i < s.length() - 1) {
                            ext = s.substring(i + 1).toLowerCase();
                        }
                        if (ext != null && ext.equals("pdb"))
                            bRet = true;
                    }
                    return bRet;
                }

                @Override
                public String getDescription() {
                    return "PDB files";
                }
            });

            if (fileDlg.showOpenDialog(m_aJmolPanel) == JFileChooser.APPROVE_OPTION) {
                File aFile = fileDlg.getSelectedFile();
                String pdb = null;
                if (aFile.exists()){
                    try{
                        BufferedReader reader = new BufferedReader(new FileReader(aFile));

                        String str = reader.readLine();
                        while (str != null){
                            pdb += str + "\n";
                            str = reader.readLine();
                        }
                        reader.close();

                    } catch (Exception ex) {
                        ex.printStackTrace();
                    }
                }
                if (pdb != null){
                    m_aJmolPanel.getViewer().openStringInline(pdb);
                    setPDBString(pdb);
                }
            }
        }
    }

    public String getDefaultPDB(){
        return Adrenochrome;
    }
    private final static String Adrenochrome =
            "HEADER    Adrenochrome: a nerve cell transmission inhibitor, and hallucinogen\n" +
            "COMPND    adrenochrome\n" +
            "AUTHOR    Created by Dave Woodcock at Okanagan University College\n" +
            "AUTHOR    email:woodcock@okanagan.bc.ca\n" +
            "AUTHOR    Date revised: Tue Aug 29 08:56:34 2000  GENERATED BY BABEL 1.6 \n" +
            "HETATM    1  C           1       0.000   0.000   0.000  1.00  0.00 \n" +
            "HETATM    2  C           1       1.400   0.000   0.000  1.00  0.00 \n" +
            "HETATM    3  C           1       2.056   1.235   0.000  1.00  0.00 \n" +
            "HETATM    4  C           1       1.351   2.436  -0.011  1.00  0.00 \n" +
            "HETATM    5  C           1      -0.047   2.437  -0.027  1.00  0.00 \n" +
            "HETATM    6  C           1      -0.714   1.206  -0.013  1.00  0.00 \n" +
            "HETATM    7  O           1      -0.597  -1.046  -0.002  1.00  0.00 \n" +
            "HETATM    8  O           1      -1.918   1.181  -0.023  1.00  0.00 \n" +
            "HETATM    9  N           1       2.243   3.571  -0.065  1.00  0.00 \n" +
            "HETATM   10  C           1       1.825   4.648   0.846  1.00  0.00 \n" +
            "HETATM   11  C           1       3.553   3.011   0.321  1.00  0.00 \n" +
            "HETATM   12  C           1       3.538   1.506  -0.050  1.00  0.00 \n" +
            "HETATM   13  H           1       4.088   0.906   0.675  1.00  0.00 \n" +
            "HETATM   14  O           1       4.023   1.295  -1.381  1.00  0.00 \n" +
            "HETATM   15  H           1       1.962  -0.934  -0.006  1.00  0.00 \n" +
            "HETATM   16  H           1      -0.608   3.370  -0.059  1.00  0.00 \n" +
            "HETATM   17  H           1       1.674   4.250   1.849  1.00  0.00 \n" +
            "HETATM   18  H           1       0.895   5.089   0.489  1.00  0.00 \n" +
            "HETATM   19  H           1       2.596   5.418   0.873  1.00  0.00 \n" +
            "HETATM   20  H           1       3.683   3.101   1.400  1.00  0.00 \n" +
            "HETATM   21  H           1       4.364   3.531  -0.190  1.00  0.00 \n" +
            "HETATM   22  H           1       4.963   1.501  -1.423  1.00  0.00 \n" +
            "CONECT    1    2    6    7\n" +
            "CONECT    1    7\n" +
            "CONECT    2    1    3   15\n" +
            "CONECT    2    3\n" +
            "CONECT    3    2    4   12\n" +
            "CONECT    4    3    5    9\n" +
            "CONECT    4    5\n" +
            "CONECT    5    4    6   16\n" +
            "CONECT    6    1    5    8\n" +
            "CONECT    6    8\n" +
            "CONECT    7    1\n" +
            "CONECT    8    6\n" +
            "CONECT    9    4   10   11\n" +
            "CONECT   10    9   17   18   19\n" +
            "CONECT   11    9   12   20   21\n" +
            "CONECT   12    3   11   13   14\n" +
            "CONECT   13   12\n" +
            "CONECT   14   12   22\n" +
            "CONECT   15    2\n" +
            "CONECT   16    5\n" +
            "CONECT   17   10\n" +
            "CONECT   18   10\n" +
            "CONECT   19   10\n" +
            "CONECT   20   11\n" +
            "CONECT   21   11\n" +
            "CONECT   22   14\n" +
            "MASTER        0    0    0    0    0    0    0    0   22    0   22    0\n" +
            "END\n";
}  

